-- File name   : EFX_VOXEL.MS
-- Title       : Voxel Generator
-- Category    : Geometry & Helper
-- Environment : 3ds max 4
-- Description : node manipulation & intersectRay experiment
-- Reference   :

-- Copyright (c) 2000-2001, EffectWare, Moon-Sun Choi
-- http://www.effectware.com
-- aeroplus@effectware.com

-- global variables -----------------------------------------------------------

global efx_floater_voxel
global efx_rollout_voxel_about
global efx_rollout_voxel_param

-- general functions ----------------------------------------------------------

-- ߽ ̿ DUMMY Ѵ.
-- cen : ceter point
-- ext : size

fn draw_center_dummy cen ext =
(
	local bbox

	bbox = dummy boxsize:[ext.x, ext.y, ext.z]
	bbox.pos = cen

	bbox
)

--  Ϳ ü 浹 θ ˻Ѵ.
-- node : test object
-- pos : ray position
-- size : size limit
-- dir : ray direction

fn check_intersect_ray node pos size dir =
(
	local hit
	local cray, nray
	local dist

	hit = false

	cray = ray pos dir
	nray = intersectRay node cray

	if nray != undefined then
	(
		dist = nray.pos - pos
		if abs(dist.x) < size.x and abs(dist.y) < size.y and abs(dist.z) < size.z then
			hit = true
	)

	hit
)

--  ũ ڽ ü 浹 θ ˻Ѵ.
-- node : test object
-- pos : box base point
-- size : box size

fn check_intersect_box node pos size =
(
	local hit

	-- X axis

	-- edge [-1, -1, -1] -> [1, -1, -1]
	hit = check_intersect_ray node [pos.x, pos.y, pos.z] size [1, 0, 0]
	if hit == true then
		return true
	-- edge [-1, -1, -1] <- [1, -1, -1]
	hit = check_intersect_ray node [pos.x + size.x, pos.y, pos.z] size [-1, 0, 0]
	if hit == true then
		return true

	-- edge [-1, 1, -1] -> [1, 1, -1]
	hit = check_intersect_ray node [pos.x, pos.y + size.y, pos.z] size [1, 0, 0]
	if hit == true then
		return true
	-- edge [-1, 1, -1] <- [1, 1, -1]
	hit = check_intersect_ray node [pos.x + size.x, pos.y + size.y, pos.z] size [-1, 0, 0]
	if hit == true then
		return true

	-- edge [-1, -1, 1] -> [1, -1, 1]
	hit = check_intersect_ray node [pos.x, pos.y, pos.z + size.z] size [1, 0, 0]
	if hit == true then
		return true
	-- edge [-1, -1, 1] <- [1, -1, 1]
	hit = check_intersect_ray node [pos.x + size.x, pos.y, pos.z + size.z] size [-1, 0, 0]
	if hit == true then
		return true

	-- edge [-1, 1, 1] -> [1, 1, 1]
	hit = check_intersect_ray node [pos.x, pos.y + size.y, pos.z + size.z] size [1, 0, 0]
	if hit == true then
		return true
	-- edge [-1, 1, 1] <- [1, 1, 1]
	hit = check_intersect_ray node [pos.x + size.x, pos.y + size.y, pos.z + size.z] size [-1, 0, 0]
	if hit == true then
		return true

	-- Y axis

	-- edge [-1, -1, -1] -> [-1, 1, -1]
	hit = check_intersect_ray node [pos.x, pos.y, pos.z] size [0, 1, 0]
	if hit == true then
		return true
	-- edge [-1, -1, -1] <- [-1, 1, -1]
	hit = check_intersect_ray node [pos.x, pos.y + size.y, pos.z] size [0, -1, 0]
	if hit == true then
		return true

	-- edge [1, -1, -1] -> [1, 1, -1]
	hit = check_intersect_ray node [pos.x + size.x, pos.y, pos.z] size [0, 1, 0]
	if hit == true then
		return true
	-- edge [1, -1, -1] <- [1, 1, -1]
	hit = check_intersect_ray node [pos.x + size.x, pos.y + size.y, pos.z] size [0, -1, 0]
	if hit == true then
		return true

	-- edge [-1, -1, 1] -> [-1, 1, 1]
	hit = check_intersect_ray node [pos.x, pos.y, pos.z + size.z] size [0, 1, 0]
	if hit == true then
		return true
	-- edge [-1, -1, 1] <- [-1, 1, 1]
	hit = check_intersect_ray node [pos.x, pos.y + size.y, pos.z + size.z] size [0, -1, 0]
	if hit == true then
		return true

	-- edge [1, -1, 1] -> [1, 1, 1]
	hit = check_intersect_ray node [pos.x + size.x, pos.y, pos.z + size.z] size [0, 1, 0]
	if hit == true then
		return true
	-- edge [1, -1, 1] <- [1, 1, 1]
	hit = check_intersect_ray node [pos.x + size.x, pos.y + size.y, pos.z + size.z] size [0, -1, 0]
	if hit == true then
		return true

	-- Z axis

	-- edge [-1, -1, -1] -> [-1, -1, 1]
	hit = check_intersect_ray node [pos.x, pos.y, pos.z] size [0, 0, 1]
	if hit == true then
		return true
	-- edge [-1, -1, -1] <- [-1, -1, 1]
	hit = check_intersect_ray node [pos.x, pos.y, pos.z + size.z] size [0, 0, -1]
	if hit == true then
		return true

	-- edge [1, -1, -1] -> [1, -1, 1]
	hit = check_intersect_ray node [pos.x + size.x, pos.y, pos.z] size [0, 0, 1]
	if hit == true then
		return true
	-- edge [1, -1, -1] <- [1, -1, 1]
	hit = check_intersect_ray node [pos.x + size.x, pos.y, pos.z + size.z] size [0, 0, -1]
	if hit == true then
		return true

	-- edge [-1, 1, -1] -> [-1, 1, 1]
	hit = check_intersect_ray node [pos.x, pos.y + size.y, pos.z] size [0, 0, 1]
	if hit == true then
		return true
	-- edge [-1, 1, -1] <- [-1, 1, 1]
	hit = check_intersect_ray node [pos.x, pos.y + size.y, pos.z + size.z] size [0, 0, -1]
	if hit == true then
		return true

	-- edge [1, 1, -1] -> [1, 1, 1]
	hit = check_intersect_ray node [pos.x + size.x, pos.y + size.y, pos.z] size [0, 0, 1]
	if hit == true then
		return true
	-- edge [1, 1, -1] <- [1, 1, 1]
	hit = check_intersect_ray node [pos.x + size.x, pos.y + size.y, pos.z + size.z] size [0, 0, -1]
	if hit == true then
		return true
)

--  ü X, Y, Z  ŭ Ѵ.
-- node : selected object
-- nx, ny, nz : division count

fn draw_voxel node nx ny nz =
(
	local ext
	local dx, dy, dz
	local bbox
	local glist = #()
	local cx, cy, cz
	local x, y, z
	local cpos
	local hit
	local nbox, nclone
	local cont

	ext = node.max - node.min
	dx = ext.x / nx
	dy = ext.y / ny
	dz = ext.z / nz

	bbox = draw_center_dummy node.center ext
	append glist bbox -- add bounding box

	cz = node.min.z
	z = 0
	while z < nz do
	(
		cy = node.min.y + dy / 2
		y = 0
		while y < ny do
		(
			cx = node.min.x + dx / 2
			x = 0
			while x < nx do
			(
				cpos = [cx, cy, cz]

				cont = progressUpdate (100 * (nx * ny * z + nx * y + x + 1) / (nx * ny * nz))
				if cont == false then
				(
					group glist name:("VOXEL_" + node.name)

					return false
				)

				--status.text = "Processing: " + ((x + 1) as string) + "," + ((y + 1) as string) + "," + ((z + 1) as string)

				hit = check_intersect_box node [cpos.x - dx / 2, cpos.y - dy / 2, cpos.z] [dx, dy, dz]
				if hit == true then
				(
					if nbox == undefined then
					(
						nbox = box width:dx length:dy height:dz pos:cpos wireColor:node.wireColor
						nbox.parent = bbox

						append glist nbox -- add original box
					)
					else
					(
						nclone = instance nbox
						nclone.pos = cpos
						nclone.wireColor = nbox.wireColor

						append glist nclone -- add cloned box
					)
				)
				cx += dx
				x += 1
			)
			cy += dy
			y += 1
		)
		cz += dz
		z += 1
	)

	group glist name:("VOXEL_" + node.name)

	true
)

-- rollout routines -----------------------------------------------------------

--
-- rollout for about
--
rollout efx_rollout_voxel_about "About"
(
	label l1 "Voxel Generator"
	label l2 "Version 1.0"
	label l3 "Choi, Moon-Sun"
	label l4 "Copyright (c) 2000-2001"
	label l5 "http://www.effectware.com"
	label l6 "aeroplus@effectware.com"
)

--
-- rollout for parameters
--
rollout efx_rollout_voxel_param "Parameters"
(
	local node, ext

	group "Node"
	(
		pickbutton pick_node "  Pick Node  "
		label node_name
		label dimx
		label dimy
		label dimz
	)

	spinner nx "X segments:" type:#integer range:[1, 100, 10]
	spinner ny "Y segments:" type:#integer range:[1, 100, 10]
	spinner nz "Z segments:" type:#integer range:[1, 100, 10]
	--label status
	button do_create " Create "

	on bounding_box_maker open do
	(
		node_name.text = "None"
	)

	on pick_node picked obj do
	(
		-- bounding box generation
		node = obj
		ext = node.max - node.min

		-- information
		node_name.text = node.name
		dimx.text = "X: " + (ext.x as string)
		dimy.text = "Y: " + (ext.y as string)
		dimz.text = "Z: " + (ext.z as string)
	)

	on do_create pressed do
	(
		if node != undefined then
		(
			escapeEnable = false
			progressStart "Processing:"

			-- division
			undo on
			(
				draw_voxel node nx.value ny.value nz.value
				--status.text = ""
			)

			progressEnd()
			escapeEnable = true
		)
	)
)

-- floater routines -----------------------------------------------------------

--  ȭ 
if efx_floater_voxel != undefined then closeRolloutFloater efx_floater_voxel
--  ȭ ߰
efx_floater_voxel = newRolloutFloater "Voxel Generator" 200 420
-- ȭڿ Ѿƿ ߰
addRollout efx_rollout_voxel_about efx_floater_voxel
addRollout efx_rollout_voxel_param efx_floater_voxel
